/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package algopher;

import java.util.*;

/** Rubik's cube moves, like R2 and F'.
 *  Find out the face or slice of a move, and check if a move is
 *  on the same slice as another move.
 *
 * @author Michiel
 */
public class Move {
    /**
     * Rubik's cube move, e.g. R2
     */
    public String value;
    private HashMap<String, String> opposites;

    /** initialize by storing opposite sides for each face
     * 
     * @param value the actual move, such as R2 or F'
     */
    public Move(String value) {
        assert(value != null);
        assert(!value.isEmpty());

        this.value = value;
        opposites = new HashMap();
        opposites.put("R","L");
        opposites.put("L","R");
        opposites.put("U","D");
        opposites.put("D","U");
        opposites.put("F","B");
        opposites.put("B","F");
    }

    /** check if this move is in the same slice as some other move.
     * E.g. R and L, or D and U2
     *
     * @param otherMove
     * @return true, if this move is on the same slice, else otherwise.
     * Also returns false if the other move is null.
     */
    public boolean inSameSlice(Move otherMove) {
        /* otherMove can actually be null on first move */
        if (otherMove == null)
            return false;
        String face = this.face();
        String other = otherMove.face();
        String opposite = this.opposite();

        return other.equals(opposite) || other.equals(face);
    }

    /** The face of a move, e.g. R for R2, F for F'
     *
     * @return the face (first char)
     */
    public String face() {
        return ""+value.charAt(0);
    }

    /** The opposite face of this move
     *
     * @return
     * R <=> L <br>
     * D <=> U <br>
     * F <=> B
     */
    public String opposite() {
        return opposites.get(this.face());
    }

    /** the move as a string
     * 
     * @return the move as a string value
     */
    @Override
    public String toString() {
        return value;
    }
}

